@@ -21,52 +21,16 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
+from typing import AsyncIterator, Type, TYPE_CHECKING
 
 from .miscmodels import try_enum, Badge
 from .iterators import PlayerIterator
 
-
-class OverrideDoc(type):
-    """Helper metaclass to make Sphinx recognise attributes from base classes.
-
-    This just overrides the object's docstring and injects any documented attributes from the base classes.
-    """
-
-    def __new__(cls, *args, **kwargs):
-        _, bases, _ = args
-        new_cls = super().__new__(cls, *args, **kwargs)
-
-        for obj in bases:
-            if obj.__doc__ is None:
-                continue
-            if new_cls.__doc__ is None:
-                continue
-            if "Attributes" not in obj.__doc__:
-                continue
-
-            try:
-                doc = obj.__doc__.split("Attributes")[1].split("\n\n")[0]
-            except (KeyError, AttributeError):
-                continue
-
-            if "Attributes" not in new_cls.__doc__:
-                new_cls.__doc__ += "\nAttributes\n----------\n" + doc.replace("----------", "")
-            else:
-                try:
-                    insert = new_cls.__doc__.index("Attributes")
-                except ValueError:
-                    return new_cls
-
-                # fmt: off
-                new_cls.__doc__ = (
-                    new_cls.__doc__[:insert + 25] + doc.replace("----------", "") + new_cls.__doc__[insert + 25:]
-                )
-                # fmt: on
-
-        return new_cls
+if TYPE_CHECKING:
+    from .players import Player
 
 
-class BaseClan(metaclass=OverrideDoc):
+class BaseClan:
     """An ABC that implements some common operations on clans, regardless of type.
 
     Attributes
@@ -111,12 +75,13 @@ class BaseClan(metaclass=OverrideDoc):
         # pylint: disable=missing-function-docstring
         return NotImplemented
 
-    def get_detailed_members(self) -> PlayerIterator:
+    def get_detailed_members(self, cls: Type["Player"] = None) -> AsyncIterator["Player"]:
         """Get detailed player information for every player in the clan.
-        This will return an AsyncIterator of :class:`Player`.
+
+        This returns a :class:`PlayerIterator` which fetches all player tags in the clan in parallel.
 
         Example
-        --------
+        ---------
 
         .. code-block:: python3
 
@@ -125,17 +90,19 @@ class BaseClan(metaclass=OverrideDoc):
             async for player in clan.get_detailed_members():
                 print(player.name)
 
-        Returns
-        -------
-        The clan members. : AsyncIterator of :class:`Player`
+
+        Yields
+        ------
+        :class:`Player`
+            A full player object of a clan member.
         """
         if self.members is NotImplemented:
             return NotImplemented
 
-        return PlayerIterator(self._client, (p.tag for p in self.members))
+        return PlayerIterator(self._client, (p.tag for p in self.members), cls=cls)
 
 
-class BasePlayer(metaclass=OverrideDoc):
+class BasePlayer:
     """An ABC that implements some common operations on players, regardless of type.
 
     Attributes
