@@ -66,6 +66,7 @@ ELIXIR_TROOP_ORDER = [
     "Miner",
     "Electro Dragon",
     "Yeti",
+    "Dragon Rider",
 ]
 
 DARK_ELIXIR_TROOP_ORDER = [
@@ -94,6 +95,7 @@ SUPER_TROOP_ORDER = [
     "Super Giant",
     "Sneaky Goblin",
     "Super Wall Breaker",
+    "Rocket Balloon",
     "Super Wizard",
     "Inferno Dragon",
     "Super Minion",
@@ -103,7 +105,9 @@ SUPER_TROOP_ORDER = [
 ]
 
 # TODO: When v2 comes around, remove siege machines from here
-HOME_TROOP_ORDER = ELIXIR_TROOP_ORDER + SUPER_TROOP_ORDER + DARK_ELIXIR_TROOP_ORDER + SIEGE_MACHINE_ORDER
+HOME_TROOP_ORDER = ELIXIR_TROOP_ORDER + DARK_ELIXIR_TROOP_ORDER + \
+                   SIEGE_MACHINE_ORDER + SUPER_TROOP_ORDER
+
 
 BUILDER_TROOPS_ORDER = [
     "Raged Barbarian",
