@@ -774,7 +774,7 @@ class EventsClient(Client):
             while self.loop.is_running():
                 end_of_season = get_season_end()
                 now = datetime.utcnow()
-                await asyncio.sleep((end_of_season - now).total_seconds() + 1)
+                await asyncio.sleep((end_of_season - now).total_seconds())
                 self.dispatch("new_season_start")
         except asyncio.CancelledError:
             pass
@@ -912,6 +912,9 @@ class EventsClient(Client):
         except KeyError:
             self._locks[key] = lock = asyncio.Lock()
 
+        if lock.locked():
+            # fast return to speed up events
+            return
         await lock.acquire()
 
         try:
@@ -950,6 +953,9 @@ class EventsClient(Client):
         except KeyError:
             self._locks[key] = lock = asyncio.Lock()
 
+        if lock.locked():
+            # fast return to speed up events
+            return
         await lock.acquire()
 
         try:
@@ -985,6 +991,9 @@ class EventsClient(Client):
         except KeyError:
             self._locks[key] = lock = asyncio.Lock()
 
+        if lock.locked():
+            # fast return to speed up events
+            return
         await lock.acquire()
 
         if self.is_cwl_active:
