@@ -21,17 +21,18 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
-import typing
 import itertools
 
+from typing import AsyncIterator, List, Optional, Type, TYPE_CHECKING
+
 from .enums import WarRound
 from .iterators import LeagueWarIterator
 from .miscmodels import try_enum, Timestamp
-from .utils import get
+from .utils import cached_property, get
 from .war_clans import WarClan, ClanWarLeagueClan
 from .war_attack import WarAttack
 
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     # pylint: disable=cyclic-import
     from .war_members import ClanWarMember  # noqa
 
@@ -89,13 +90,13 @@ class ClanWar:
     def _from_data(self, data: dict) -> None:
         data_get = data.get
 
-        self.state = data_get("state")
+        self.state: str = data_get("state")
         self.preparation_start_time = try_enum(Timestamp, data=data_get("preparationStartTime"))
         self.start_time = try_enum(Timestamp, data=data_get("startTime"))
         self.end_time = try_enum(Timestamp, data=data_get("endTime"))
-        self.war_tag = data_get("tag")
+        self.war_tag: str = data_get("tag")
 
-        self.team_size = data_get("teamSize") or len(data_get("clan", {}).get("members", []))
+        self.team_size: int = data_get("teamSize") or len(data_get("clan", {}).get("members", []))
 
         clan_data = data_get("clan")
         # annoying bug where if you request a war with a clan tag that clan could be the opponent or clan,
@@ -109,17 +110,17 @@ class ClanWar:
             self.opponent = try_enum(WarClan, data=clan_data, client=self._client, war=self)
 
     @property
-    def attacks(self) -> typing.List[WarAttack]:
+    def attacks(self) -> List[WarAttack]:
         """List[:class:`WarAttack`]: Returns all attacks this war, sorted by attack order."""
         return sorted([*self.clan.attacks, *self.opponent.attacks], key=lambda x: x.order, reverse=True)
 
     @property
-    def members(self) -> typing.List["ClanWarMember"]:
+    def members(self) -> List["ClanWarMember"]:
         """List[:class:`ClanWarMember`]: A list of members that are in the war."""
         return sorted([*self.clan.members, *self.opponent.members], key=lambda x: (not x.is_opponent, x.map_position))
 
     @property
-    def type(self) -> typing.Optional[str]:
+    def type(self) -> Optional[str]:
         """:class:`str`: Returns either ``friendly``, ``random`` or ``cwl``.
 
         This will returns ``None`` if the clan is not in war, or ``cwl`` if the clan is in a league war.
@@ -201,7 +202,7 @@ class ClanWar:
         """:class:`bool`: Returns a boolean indicating if the war is a Clan War League (CWL) war."""
         return self.type == "cwl"
 
-    def get_member(self, tag: str) -> typing.Optional["ClanWarMember"]:
+    def get_member(self, tag: str) -> Optional["ClanWarMember"]:
         """Return a :class:`ClanWarMember` with the tag provided. Returns ``None`` if not found.
 
         Example
@@ -223,7 +224,7 @@ class ClanWar:
         away_member = self.opponent.get_member(tag)
         return away_member
 
-    def get_member_by(self, **attrs):
+    def get_member_by(self, **attrs) -> Optional["ClanWarMember"]:
         """Returns the first :class:`WarMember` that meets the attributes passed
 
         This will return the first member matching the attributes passed.
@@ -238,7 +239,7 @@ class ClanWar:
         """
         return get(self.members, **attrs)
 
-    def get_attack(self, attacker_tag: str, defender_tag: str) -> typing.Optional[WarAttack]:
+    def get_attack(self, attacker_tag: str, defender_tag: str) -> Optional[WarAttack]:
         """Return the :class:`WarAttack` with the attacker tag and defender tag provided.
 
         If the attack was not found, this will return ``None``.
@@ -255,7 +256,7 @@ class ClanWar:
             return None
         return get(attacks, defender_tag=defender_tag)
 
-    def get_defenses(self, defender_tag: str) -> typing.List[WarAttack]:
+    def get_defenses(self, defender_tag: str) -> List[WarAttack]:
         """Return a :class:`list` of :class:`WarAttack` for the defender tag provided.
 
         If the player has no defenses, this will return an empty list.
@@ -310,9 +311,9 @@ class ClanWarLogEntry:
     def _from_data(self, data: dict) -> None:
         data_get = data.get
 
-        self.result = data_get("result")
+        self.result: str = data_get("result")
         self.end_time = try_enum(Timestamp, data=data_get("endTime"))
-        self.team_size = data_get("teamSize")
+        self.team_size: int = data_get("teamSize")
 
         self.clan = self._fake_load_clan(data_get("clan"))
         self.opponent = self._fake_load_clan(data_get("opponent"))
@@ -352,7 +353,7 @@ class ClanWarLeagueGroup:
 
     """
 
-    __slots__ = ("state", "season", "rounds", "number_of_rounds", "_client", "__iter_clans", "_clans")
+    __slots__ = ("state", "season", "rounds", "number_of_rounds", "_client", "__iter_clans", "_cs_clans")
 
     def __repr__(self):
         attrs = [
@@ -363,37 +364,31 @@ class ClanWarLeagueGroup:
 
     def __init__(self, *, data, client, **_):
         self._client = client
-        self._clans = []
         self._from_data(data)
 
     def _from_data(self, data: dict) -> None:
         data_get = data.get
 
-        self.state = data_get("state")
-        self.season = data_get("season")
+        self.state: str = data_get("state")
+        self.season: str = data_get("season")
 
         rounds = data_get("rounds")
-        self.number_of_rounds = len(rounds)
+        self.number_of_rounds: int = len(rounds)
         # the API returns a list and the rounds that haven't started contain war tags of #0 (not sure why)...
         # we want to get only the valid rounds
-        self.rounds = [n["warTags"] for n in rounds if n["warTags"][0] != "#0"]
+        self.rounds: List[List[str]] = [n["warTags"] for n in rounds if n["warTags"][0] != "#0"]
 
         self.__iter_clans = (ClanWarLeagueClan(data=data, client=self._client) for data in data_get("clans", []))
 
-    @property
-    def clans(self) -> typing.List[ClanWarLeagueClan]:
+    @cached_property("_cs_clans")
+    def clans(self) -> List[ClanWarLeagueClan]:
         """List[:class:`LeagueClan`]: Returns all participating clans."""
-        clans = self._clans
-        if clans:
-            return clans
+        return list(self.__iter_clans)
 
-        self._clans = clans = list(self.__iter_clans)
-        return clans
-
-    def get_wars_for_clan(self, clan_tag: str, cls: typing.Type[ClanWar] = ClanWar):
+    def get_wars_for_clan(self, clan_tag: str, cls: Type[ClanWar] = ClanWar) -> AsyncIterator[ClanWar]:
         """Returns every war the clan has participated in this current CWL.
 
-        This will return an AsyncIterator of :class:`ClanWar`.
+        This returns a :class:`LeagueWarIterator` which fetches all wars in parallel.
 
         Example
         --------
@@ -412,18 +407,19 @@ class ClanWarLeagueGroup:
         cls: Type[:class:`ClanWar`]: The constructor used to create the league war.
                                      This should inherit from :class:`ClanWar`.
 
-        Returns
-        ---------
-        All wars in the given round. : AsyncIterator of :class:`ClanWar`
+        Yields
+        ------
+        :class:`ClanWar`
+            A war in the current CWL season with the clan in it..
         """
         return LeagueWarIterator(client=self._client, tags=itertools.chain(*self.rounds), clan_tag=clan_tag, cls=cls)
 
     def get_wars(
-        self, cwl_round=WarRound.current_war, cls: typing.Type[ClanWar] = ClanWar
-    ) -> LeagueWarIterator:
+        self, cwl_round: WarRound = WarRound.current_war, cls: Type[ClanWar] = ClanWar
+    ) -> AsyncIterator[ClanWar]:
         """Returns war information for every war in a league round.
 
-        This will return an AsyncIterator of :class:`ClanWar`.
+        This returns a :class:`LeagueWarIterator` which fetches all wars in parallel.
 
         Example
         --------
@@ -446,10 +442,10 @@ class ClanWarLeagueGroup:
             This defaults to ``coc.WarRound.current_war``.
 
 
-        Returns
-        ----------
-
-        All wars in the given round. : AsyncIterator of :class:`ClanWar`
+        Yields
+        ------
+        :class:`ClanWar`
+            A war in the requested round.
         """
         is_prep = self.state == "preparation"
         num_rounds = len(self.rounds)
